#!/bin/sh
#
# Component: archive
#
# Description:
#  deletes a study from the archive after checking that it is successfully mreged back to the worklist.
#
# Usage:
#   onStudyMerged.sh -t <type> -e <event> -i <id>
#
#    -t type  : Object type to be released. It will always be "ARCHIVEMERGE"
#    -e event : Event triggered running this script. It will always be "RELEASED"
#    -i id    : Identifies the studydb object (=Study Instance UID)
#
# Last Updated     : $Author $
# Update Date Time : $Date $
# RCS File Name    : $RCSfile $
# RCS Revision     : $Revision $
# Locked by        : $Locker $
# Symbolic name    : $Name $
# State            : $State $
#

. $HOME/etc/setenv.sh

# if delete.sh is called directly:
# run as a task instead
if [ `basename $0` = "onStudyMerged.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi

LOGGER="/usr/bin/logger"
TAG="archive/onStudyMerged[$$]"

study=""
type=""
event=""
errMsg=""

while getopts 'e:t:i:' OPT; do
	case $OPT in
	e) event="$OPTARG" ;;
	t) type="$OPTARG" ;;
	i) study="$OPTARG" ;;
	*) errMsg="Invalid_Parameter:$OPT" ;;
	esac
done

if [ "$type" != "ARCHIVEMERGE" -o "$event" != "RELEASED" -o -z "$study" -o -n "$errMsg" ] ; then
	$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Invalid argument: Type: $type; Event: $event; Study: $study; $errMsg"
	exit 1;
fi

# include nettask.shi for checking if the device is temporary down
. $TASKD_HOME/nettask.shi

# override getPeer_() of nettask.shi
getPeer_()
{
	echo "@Archive @Archive-PUT"
}

run_()
{
	if [ -z "$REPOSITORY_DICOM_DATA" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Missing environment"
		return 1
	fi

	if [ ! -f "$ARCHIVE_VAR/archive.cfg" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Missing archive.cfg"
		exit 1
	fi

	. "$ARCHIVE_VAR/archive.cfg"
	if [ "$Archive" != "readonly" -o "$DoMerge" != "true" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Archive is not in merge mode "
		exit 1
	fi

	if [ -z "$study" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Missing parameter"
		return 125
	fi

	SELF_SERVER_ID=""
	[ -f "$CONF/serverid.cfg" ] && . $CONF/serverid.cfg
	if [ -z "$SELF_SERVER_ID" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Self server id is not set in $CONF/serverid.cfg"
		return 1
	fi

	$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "delete study: $study"

	# Check if the archive is temporarily down and return
	checkPeer_
	if [ $? -ne 0 ] ; then
		$LOGGER -p $LOG_FACILITY.error -t "$TAG" "Archive delete is temporarily unavailable - retry"
		return 1
	fi

	archDir=`echo "select ARCHDIR from $ARCHIVETBL where STYIUID='$study';" | \
			$MYSQL_HOME/bin/mysql -N $DBNAME`

	if [ $? -ne 0 ] ; then
		# MySQL Error
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "MySQL error occured while retrieving archive directory for $study"
		return 1
	fi

	if [ -z "$archDir" ] ; then
		# Missing study or archive Dir from Archive database
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Missing study or archive Dir from Archive database"
		return 125
	fi

	studyArchDir="$ARCHIVE_DIR/$archDir/$study"
	$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "Archdir for $study is ${studyArchDir}"

	studyFiles=`ls ${studyArchDir}`
	if [ $? -ne 0 ] ; then
		# can't ls studyArchDir, Archive is probably unmounted
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Could not 'ls' ${studyArchDir}, Archive is probably umounted"
		return 1
	fi

	# check whether the file is already there
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`

	for i in $studyFiles ; do
		if [ ! -s "${studyArchDir}/$i" ] ; then
			# The file has 0 size
			$LOGGER -p $LOG_FACILITY.error -t "$TAG" "${studyArchDir}/$i has a size of zero bytes"
			$DRWATSON_HOME/addmessage.sh -i "zero_arch_$study" -t "WARNING" -l "TMP" "Zero byte size object(s) in the archive for $study"
			continue
		fi

		if [ -f $studydir/$i ] ; then
			#IgnoreTagsFile="${ARCHIVE_HOME}/ignoretags"
			#[ -f "${ARCHIVE_VAR}/ignoretags" ] && IgnoreTagsFile="${ARCHIVE_VAR}/ignoretags"
			#$DICOM_HOME/bin/objdiff.sh -d -i ${IgnoreTagsFile} ${studydir}/$i ${studyArchDir}/$i 2>/dev/null
			pbrtag1=""
			pbrtag2=""
			if [ "${i}" = "${i#PbR}" ] ; then
				#not a PbR file
				pbr1=`ls -1 $studydir/PbR*` 2>/dev/null
				if [ $? -eq 0 -a `echo "$pbr1"|wc -l` -eq 1 ] ; then
					pbrtag1="-1 $pbr1"
				fi
				pbr2=`ls -1 $studyArchDir/PbR*` 2>/dev/null
				if [ $? -eq 0 -a `echo "$pbr2"|wc -l` -eq 1 ] ; then
					pbrtag2="-2 $pbr2"
				fi
			fi
			error=`$UTILS_HOME/bin/objdiff $pbrtag1 $pbrtag2 ${studydir}/${i} ${studyArchDir}/${i}`
			
			if [ $? -ne 0 ] ; then
				# the file is already there but does not match
				$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "Delete object ${studyArchDir}/$i failed, different object already there ($error)"
				return 1
			fi
		else
			$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "Delete object ${studyArchDir}/$i failed, object missing from worklist"
			return 1
		fi
	done

	echo "I think deleting this study is OK"
	oldpwd=`pwd`
	cd $studyArchDir
	cd ..
	rm -rf $studyArchDir
	echo "delete from $ARCHIVETBL where STYIUID='$study';" | $MYSQL_HOME/bin/mysql -N $DBNAME
	# now we are in the parent, if it's empty we may delete it.
	entryCount=`ls -1| wc -l`
	if [ $entryCount -eq 0 ] ; then
		parent=`pwd`
		cd ..
		rm -rf $parent
		# now we are in the grandparent, if it's empty we may delete it.
		entryCount=`ls -1| wc -l`
		if [ $entryCount -eq 0 ] ; then
			parent=`pwd`
			cd ..
			rm -rf $parent
			# we will not need to remove more than two levels
		fi
	fi
	cd $oldpwd
	return 0
}


# override print_
print_()
{
	MSG="Delete study $study from the archive"
	echo $MSG
}

#
# Print value to be displayed on the WEB GUI
#
get_display_name_()
{
	echo "Delete study from legacy EP archive after merge to enterprise archive" 
}

#
# Print resources used by the task 
#
get_resources_()
{
	echo "System.NET.@Archive.PUT"
}

#
# Print property value
#
get_property_()
{
	if [ "$PROPERTY" = "1PST" ] ; then
		echo -n "$study"
		return 0
	fi
	 
	if [ "$PROPERTY" = "1PPE" ] ; then
		echo -n "@Archive" 
		return 0
	fi

	get_def_property_
}


get_watson_message_()
{
	[ -z "$DR_WATSON_ID" ] && DR_WATSON_ID="archivedelete.$study"
	[ -z "$DR_WATSON_MESSAGE" ] && DR_WATSON_MESSAGE="Deleting $study from (Internal) Archive"

	if [ "$RUN_RESULT" -gt 124 ] ; then
		WATSON_MESSAGE_ID="$DR_WATSON_ID.failed"
		WATSON_MESSAGE_TEXT="FAILED: $DR_WATSON_MESSAGE"
		return
	fi

	if [ "$RUN_RESULT" -ne 0 -a "$RETRY_NUM" -gt 0 -a $(( $RETRY_NUM % 10)) -eq 0 ] ; then
		WATSON_MESSAGE_TEXT="ERROR: $DR_WATSON_MESSAGE [Retry #$RETRY_NUM]"
		WATSON_MESSAGE_ID="$DR_WATSON_ID.$RETRY_NUM"
	fi
} # get_watson_message_()

